package org.verohallinto.apitamoclient.ui;

import org.apache.pivot.collections.ArrayList;
import org.apache.pivot.collections.Sequence;
import org.apache.pivot.collections.immutable.ImmutableList;
import org.apache.pivot.util.Filter;
import org.apache.pivot.wtk.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.verohallinto.apitamoclient.util.Props;
import org.verohallinto.apitamoclient.util.Utils;

import java.io.File;
import java.io.IOException;

/**
 * Send tab.
 */
public class TabSend {

    private static final Logger log = LoggerFactory.getLogger(TabSend.class);
    private ApitamopkiClientUI ui;

    /**
     * Constuctor.
     * </p>
     *
     * @param ui <code>ApitamopkiClientUI</code>
     */
    public TabSend(final ApitamopkiClientUI ui) {

        this.ui = ui;
        // Lhetettvn aineiston poiminta
        ui.materialBrowser = new FileBrowserSheet();
        ui.materialBrowser.setMode(FileBrowserSheet.Mode.OPEN);
        ui.materialBrowser.setRootDirectory(ui.workDir);

        if (Utils.isEmpty(ui.emails.getText())) {
            // Oletus shkpostiosoite Windows-tunnuksen mukaan
            ui.emails.setText(Props.getEmail(ui.winUser));
        }

        ui.materialBrowser.setDisabledFileFilter(new Filter<File>() {
            @Override
            public boolean include(final File item) {
                String ext = ui.action.getExtension(item.getName());

                return !(item.isDirectory() || ".TXT".equalsIgnoreCase(ext) || ".XML".equalsIgnoreCase(ext));
            }
        });

        // Lhetettvn aineston valintanappi
        ui.materialPB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(Button button) {
                ui.materialBrowser.open(ui, new SheetCloseListener() {
                    @Override
                    public void sheetClosed(final Sheet sheet) {

                        if (sheet.getResult()) {
                            final Sequence<File> selectedFiles = ui.materialBrowser.getSelectedFiles();
                            final ListView listView = new ListView();
                            listView.setListData(new ArrayList<>(selectedFiles));
                            ui.material.setText(selectedFiles.get(0).getAbsolutePath());
                        }
                    }
                });
            }
        });

        // Aineiston drag & drop
        ui.material.setDropTarget(new DropMaterial() {
            @Override
            public DropAction dragEnter(Component component, Manifest dragContent, int supportedDropActions, DropAction userDropAction) {
                return acceptMaterial(dragContent, supportedDropActions);
            }

            @Override
            public DropAction dragMove(Component component, Manifest dragContent, int supportedDropActions, int x, int y, DropAction userDropAction) {
                return acceptMaterial(dragContent, supportedDropActions);
            }

            @Override
            public DropAction drop(Component component, Manifest dragContent, int supportedDropActions, int x, int y, DropAction userDropAction) {
                DropAction dropAction = null;

                if (dragContent.containsFileList()) {
                    try {
                        ui.material.setText(dragContent.getFileList().get(0).getAbsolutePath());
                        dropAction = DropAction.COPY;
                    } catch (IOException e) {
                        ui.action.handleException(e);
                    }
                }

                dragExit(component);

                return dropAction;
            }
        });

        // Ilmoitusten kieli
        ui.sendLanguageBG.getButtonGroupListeners().add(new ButtonGroupListener.Adapter() {
            @Override
            public void selectionChanged(final ButtonGroup bg, final Button previousSelection) {
                super.selectionChanged(bg, previousSelection);

                final Button button = bg.getSelection();
                final Object data = button.getButtonData();

                for (Button lang : ui.retrLanguageBG) {
                    if (lang.getButtonData().equals(data)) {
                        lang.setSelected(true);
                        break;
                    }
                }
            }
        });

        // Taustaksittely
        ui.backgroundCB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(final Button button) {

                if (button.isSelected() && Utils.isEmpty(ui.emails.getText())) {
                    // Oletus shkpostiosoite Windows-tunnuksen mukaan
                    ui.emails.setText(Props.getEmail(ui.winUser));
                }
            }
        });

        // Lhetettvien liitteiden poiminta
        ui.attachBrowser = new FileBrowserSheet();
        ui.attachBrowser.setMode(FileBrowserSheet.Mode.OPEN_MULTIPLE);
        ui.attachBrowser.setRootDirectory(ui.workDir);
        ui.attachBrowser.setDisabledFileFilter(new Filter<File>() {
            @Override
            public boolean include(final File item) {
                String ext = ui.action.getExtension(item.getName());

                return !(item.isDirectory() || ".PDF".equalsIgnoreCase(ext));
            }
        });

        // Liitteiden drag & drop
        ui.attachLV.setDropTarget(new DropMaterial() {
            @Override
            public DropAction dragEnter(Component component, Manifest dragContent, int supportedDropActions, DropAction userDropAction) {
                return acceptAttachment(dragContent, supportedDropActions);
            }

            @Override
            public DropAction dragMove(Component component, Manifest dragContent, int supportedDropActions, int x, int y, DropAction userDropAction) {
                return acceptAttachment(dragContent, supportedDropActions);
            }

            @Override
            public DropAction drop(Component component, Manifest dragContent, int supportedDropActions, int x, int y, DropAction userDropAction) {
                DropAction dropAction = null;

                if ("COPY".equals(acceptAttachment(dragContent, supportedDropActions).name())) {
                    try {
                        for (File file : dragContent.getFileList()) {
                            String name = file.getName().toUpperCase();

                            if (name.endsWith(".PDF")) {
                                int i = ((ArrayList<File>)ui.attachLV.getListData()).add(file);
                                ui.attachLV.setItemChecked(i, true);
                            }
                        }

                        dropAction = DropAction.COPY;
                    } catch (IOException e) {
                        ui.action.handleException(e);
                    }
                }

                dragExit(component);

                return dropAction;
            }
        });

        // Lhetettvien liitteiden valintanappi
        ui.attachPB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(Button button) {

                ui.attachBrowser.open(ui, new SheetCloseListener() {
                    @Override
                    public void sheetClosed(final Sheet sheet) {

                        if (sheet.getResult()) {
                            final ImmutableList<File> valitut = ui.attachBrowser.getSelectedFiles();

                            if (!valitut.isEmpty()) {
                                if (!ui.attachLV.getListData().isEmpty()) {
                                    for (int x = 0; x < valitut.getLength(); x++) {
                                        int y = ((ArrayList<File>)ui.attachLV.getListData()).add(valitut.get(x));
                                        ui.attachLV.setItemChecked(y, true);
                                    }
                                } else {
                                    ui.attachLV.setListData(new ArrayList<>(valitut));

                                    for (int x = 0; x < valitut.getLength(); x++) {
                                        ui.attachLV.setItemChecked(x, true);
                                    }
                                }
                            }
                        }
                    }
                });
            }
        });

        // Liitetiedostojen tyhjennys
        ui.attachClearPB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(final Button button) {
                ui.attachLV.getListData().clear();
            }
        });

        // Tarkastustuloksen hakemiston poiminta
        ui.resultBrowser = new FileBrowserSheet();
        ui.resultBrowser.setMode(FileBrowserSheet.Mode.SAVE_TO);
        ui.resultBrowser.setRootDirectory(ui.workDir);
        ui.resultBrowser.setDisabledFileFilter(new Filter<File>() {

            @Override
            public boolean include(final File item) {

                return !(item.isDirectory());
            }
        });

        // Tarkastuksen tuloksen hakemisto
        ui.sendResult.setText(ui.workDir.getAbsolutePath());

        // Tarkastustuloksen hakemiston pominta
        ui.sendResultPB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(Button button) {

                ui.resultBrowser.open(ui, new SheetCloseListener() {
                    @Override
                    public void sheetClosed(final Sheet sheet) {

                        if (sheet.getResult()) {
                            final Sequence<File> selectedFiles = ui.resultBrowser.getSelectedFiles();
                            final ListView listView = new ListView();
                            listView.setListData(new ArrayList<>(selectedFiles));
                            ui.sendResult.setText(selectedFiles.get(0).getAbsolutePath());
                        }
                    }
                });
            }
        });

        // Aineiston lhetysnappi
        ui.sendPB.getButtonPressListeners().add(new ButtonPressListener() {
            @Override
            public void buttonPressed(final Button button) {

                ui.resLeft.setText("");
                ui.resRight.setText("");
                ui.status.setText("");
                ui.status.repaint(true);
                ui.sendPB.setEnabled(false);
                ui.sendPB.repaint(true);

                try {
                    ui.action.sendOperation();
                } catch (Exception e) {
                    ui.action.handleException(e);
                }

                ui.sendPB.setEnabled(true);
                ui.sendPB.repaint(true);
            }
        });
    }

    /**
     * Aineiston hyvksynt (drag & drop).
     * </p>
     *
     * @param dragContent          <code>Manifest</code>
     * @param supportedDropActions <code>int</code>
     * @return DropAction (COPY tai null).
     */
    private DropAction acceptMaterial(final Manifest dragContent, int supportedDropActions) {

        if (log.isDebugEnabled()) {
            log.debug("acceptMaterial() - begin.");
        }

        DropAction dropAction = null;

        if (dragContent.containsFileList() && DropAction.COPY.isSelected(supportedDropActions)) {
            try {
                if (dragContent.getFileList().getLength() == 1) {
                    String name = dragContent.getFileList().get(0).getName().toUpperCase();

                    if (name.endsWith(".TXT") || name.endsWith(".XML")) {
                        dropAction = DropAction.COPY;
                    }
                }
            } catch (IOException e) {
                ui.action.handleException(e);
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("acceptMaterial() - end. action=" + (dropAction != null ? dropAction.name() : "null"));
        }

        return dropAction;
    }

    /**
     * Liitteiden hyvksynt (drag & drop).
     * </p>
     *
     * @param dragContent          <code>Manifest</code>
     * @param supportedDropActions <code>int</code>
     * @return DropAction (COPY tai null).
     */
    private DropAction acceptAttachment(final Manifest dragContent, int supportedDropActions) {

        if (log.isDebugEnabled()) {
            log.debug("acceptAttachment() - begin.");
        }

        DropAction dropAction = null;

        if (dragContent.containsFileList() && DropAction.COPY.isSelected(supportedDropActions)) {
            try {
                if (!dragContent.getFileList().isEmpty()) {
                    for (File file : dragContent.getFileList()) {
                        String name = file.getName().toUpperCase();

                        if (name.endsWith(".PDF")) {
                            dropAction = DropAction.COPY;
                            break;
                        }
                    }
                }
            } catch (IOException e) {
                ui.action.handleException(e);
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("acceptAttachment() - end. action=" + (dropAction != null ? dropAction.name() : "null"));
        }

        return dropAction;
    }
}
